#include <qcheckbox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qvbox.h>
#include <qhbox.h>

#include <kdebug.h>
#include <kdialogbase.h>
#include <klocale.h>

#include "kforms.h"

namespace KForms {

Builder::Builder()
    : dlg(0), parent(0)
{
}

Builder::~Builder()
{
}

bool Builder::load( const QString &filename )
{
    QFile f( filename );

    if ( !f.open( IO_ReadOnly ) )
	return false;

    if ( !doc.setContent( &f ) ) {
      f.close();
      return false;
    }
    f.close();

    walkDocument();
    return true;
}

void Builder::walkDocument()
{
    QDomElement root = doc.documentElement();
    processElement( root );
    walkElement( root );
}

void Builder::walkElement( const QDomElement &elem )
{
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    processElement( e );
	    walkElement( e );
	}
	n = n.nextSibling();
    }
}

void Builder::processElement( const QDomElement &elem )
{
    QString tag = elem.tagName();
    if ( tag == "dialog" )
	processDialog( elem );
    else if ( tag == "page" )
	processPage( elem );
    else if ( tag == "form" )
	processForm( elem );
    else if ( tag == "label" )
	processLabel( elem );
    else if ( tag == "field" )
	processField( elem );
    else if ( tag == "spacer" )
	processSpacer( elem );
}

QString Builder::findText( const QDomElement &elem, const QString &tag )
{
    QString text;
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if ( (!e.isNull()) && ( e.tagName() == tag )) {
	    text = e.text();
	    break;
	}
	n = n.nextSibling();
    }

    return text;
}

int Builder::processButtons( const QDomElement &elem, KDialogBase::ButtonCode &def )
{
    int buttons = 0;
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if ( e.tagName() == "button" ) {
	    bool defFlag = ( e.attribute("default") == "1" );
	    KDialogBase::ButtonCode id = buttonId( e.attribute("name") );

	    buttons = buttons | id;
	    if ( defFlag || ( def == 0 ) )
		def = id;
	}
	n = n.nextSibling();
    }

    buttons = buttons ? buttons : KDialogBase::Close;
    def = def ? def : KDialogBase::Close;

    return buttons;
}

KDialogBase::ButtonCode Builder::buttonId( const QString &name )
{
    KDialogBase::ButtonCode id = static_cast<KDialogBase::ButtonCode>( 0 );
    if ( name == "ok" )
	id =  KDialogBase::Ok;
    else if ( name == "cancel" )
	id = KDialogBase::Cancel;
    else if ( name == "help" )
	id = KDialogBase::Help;
    else if ( name == "apply" )
	id = KDialogBase::Apply;
    else if ( name == "yes" )
	id = KDialogBase::Yes;
    else if ( name == "no" )
	id = KDialogBase::No;
    else if ( name == "close" )
	id = KDialogBase::Close;

    return id;
}

void Builder::processDialog( const QDomElement &elem )
{
    QString type = elem.attribute( "type" );

    int face = 0;
    if ( type == "treelist" )
	face = KDialogBase::TreeList;
    else if ( type == "iconlist" )
	face = KDialogBase::IconList;
    else if ( type == "tabbed" )
	face = KDialogBase::Tabbed;
    else if ( type == "plain" )
	face = KDialogBase::Plain;

    KDialogBase::ButtonCode def = static_cast<KDialogBase::ButtonCode>( 0 );
    int buttons = processButtons( elem, def );

    QString title = findText( elem, QString("title") );
    if ( title.isNull() )
	title = i18n("KForms");

    parent = dlg = new KDialogBase( face, title, buttons, def );
}

void Builder::processPage( const QDomElement &elem )
{
    QString title = findText( elem, QString("title") );
    QString name = elem.attribute( "name" );

    parent = dlg->addVBoxPage( name, title );
}

void Builder::processForm( const QDomElement &elem )
{
    QString title = findText( elem, QString("title") );
    QString name = elem.attribute( "name" );
}

void Builder::processLabel( const QDomElement &elem )
{
    QString name = elem.attribute( "name" );
    QString text = elem.text();
    new QLabel( text, parent, name.local8Bit() );
}

void Builder::processSpacer( const QDomElement &/*elem*/ )
{
    QWidget *s = new QWidget( parent );
    s->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
}

void Builder::processField( const QDomElement &elem )
{
    QString name = elem.attribute( "name" );
    QString type = elem.attribute( "type", "string" );
    QString title = findText( elem, QString("title") );
    QString value = findText( elem, QString("value") );

    QHBox *field = new QHBox( parent, name.local8Bit() );
    field->setSpacing( dlg->spacingHint() );

    QLabel *label = new QLabel( title, field );
    label->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );

    if ( type == "int" ) {
	QSpinBox *sp = new QSpinBox( field );
	sp->setValue( value.toInt() );
    }
    else if ( type == "string" ) {
	new QLineEdit( value, field );
    }
    else if ( type == "filename" ) {
	new QLineEdit( value, field );
	QPushButton *pb = new QPushButton( "Browse...", field );
	pb->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
    }
    else if ( type == "bool" ) {
	QCheckBox *cb = new QCheckBox( field );
	cb->setText( title );
	if ( value == "true" )
	    cb->setChecked( true );
	else if ( value == "false" )
	    cb->setChecked( false );
    }
    else {
	new QLineEdit( value, field );
    }
}

}; // namespace KForms

// Local Variables:
// c-basic-offset: 4
// End:



